﻿using Microsoft.TeamFoundation.Build.WebApi;
using Microsoft.VisualStudio.Services.Common;
using System;
using System.Net;
using System.Threading.Tasks;


namespace TfsCloneBuildDef
{
    class Program
    {
        static void Main(string[] args)
        {
            MainAsync(args).Wait();
        }

        public static async Task MainAsync(string[] args)
        {
            try
            {
                if (args.Length != 6)
                {
                    WriteUsage();
                    return;
                }

                var collectionUrl = args[0];
                var username = args[1];
                var password = args[2];
                var sourceProject = args[3];
                var destinationProject = args[4];
                var buildDef = int.Parse(args[5]);

                await CloneBuildDefAsync(collectionUrl, buildDef, sourceProject, destinationProject, username, password);
            }
            catch (Exception ex)
            {
                Console.Error.WriteLine(ex);
                WriteUsage();
            }
        }

        private static void WriteUsage()
        {
            Console.WriteLine("Clones TFS 2015 build definition from one team project to another");
            Console.WriteLine("Usage: TfsCloneBuildDef.exe CollectionUrl Username Password SourceProject TargetProject BuildDefinitionID");
        }

        private static async Task CloneBuildDefAsync(string collectionURL, int buildDefId, string sourceProj, string targetProj, string username, string password)
        {
            var cred = new VssCredentials(new WindowsCredential(new NetworkCredential(username, password)));
            var buildClient = new BuildHttpClient(new Uri(collectionURL, UriKind.Absolute), cred);

            var buildDef = (await buildClient.GetDefinitionAsync(sourceProj, buildDefId)) as BuildDefinition;

            buildDef.Project = null;
            buildDef.Name += "_clone";

            await buildClient.CreateDefinitionAsync(buildDef, targetProj);

            Console.WriteLine($"Succesfully cloned build definition '{buildDef.Name}' from project '{sourceProj}' to project '{targetProj}'.");

        }
    }
}
