﻿using Microsoft.TeamFoundation.Build.WebApi;
using Microsoft.VisualStudio.Services.Common;
using Newtonsoft.Json;
using System;
using System.IO;
using System.Linq;
using System.Net;
using System.Threading.Tasks;

namespace TfsExportBuildDef
{
    class Program
    {
        static void Main(string[] args)
        {
            MainAsync(args).Wait();
        }

        public static async Task MainAsync(string[] args)
        {
            try
            {
                if (args.Length < 6 || args.Length > 7)
                {
                    WriteUsage();
                    return;
                }

                var collectionUrl = args[0];
                var username = args[1];
                var password = args[2];
                var project = args[3];
                var filePath = args[4];
                var buildDef = args[5];

                if (args.Last().ToLower() == "/i")
                    await ImportBuildDef(collectionUrl, username, password, project, filePath, buildDef);
                else
                    await ExportBuildDef(collectionUrl, username, password, project, filePath, int.Parse(buildDef));
            }
            catch (Exception ex)
            {
                Console.Error.WriteLine(ex);
                WriteUsage();
            }
        }

        private static string projectReplacement = "#project_name#";

        private static async Task ExportBuildDef(string collectionUrl, string username, string password, string project, string filePath, int buildDefId)
        {
            var buildClient = createClient(collectionUrl, username, password);

            //we export build definition
            Console.WriteLine($"Exporting build definition ID:{buildDefId} from '{project}' project.");
            var buildDef = (await buildClient.GetDefinitionAsync(project, buildDefId)) as BuildDefinition;
            buildDef.Project = null;
            var json = JsonConvert.SerializeObject(buildDef);
            json = json.Replace($"\"{project}\"", $"\"{projectReplacement}\"");
            json = json.Replace($"$/{project}", $"$/{projectReplacement}");

            File.WriteAllText(filePath, json);
            Console.WriteLine($"Build definition '{buildDef.Name}' succesfully exported to '{filePath}'.");
        }



        private static async Task ImportBuildDef(string collectionUrl, string username, string password, string project, string filePath, string newBuildName)
        {
            var buildClient = createClient(collectionUrl, username, password);
            //we import build definition
            if (!File.Exists(filePath))
                throw new FileNotFoundException("File does not exist!", filePath);
            Console.WriteLine($"Importing build definition from file '{filePath}' to '{project}' project.");
            var json = File.ReadAllText(filePath);
            json = json.Replace(projectReplacement, project);
            var buildDef = JsonConvert.DeserializeObject<BuildDefinition>(json);
            buildDef.Name = newBuildName;
            await buildClient.CreateDefinitionAsync(buildDef, project);
            Console.WriteLine($"Build definition '{buildDef.Name}' succesfully imported to '{project}' project.");
        }

        private static BuildHttpClient createClient(string collectionUrl, string username, string password)
        {
            var cred = new VssCredentials(new WindowsCredential(new NetworkCredential(username, password)));
            return new BuildHttpClient(new Uri(collectionUrl, UriKind.Absolute), cred);
        }

        private static void WriteUsage()
        {
            Console.WriteLine("Export/Import TFS 2015 build definition to/from JSON file.");
            Console.WriteLine("Usage export: TfsExportBuildDef.exe CollectionUrl Username Password ProjectName FilePath BuildDefinitionID");
            Console.WriteLine("Usage import: TfsExportBuildDef.exe CollectionUrl Username Password ProjectName FilePath NewBuildDefName /i");
        }
    }
}
